/**
 * @copyright 2017 Tridium, Inc. All Rights Reserved.
 * @author Logan Byam
 */

define(['baja!',
        'lex!baja'], function (
         baja,
         lexs) {

  'use strict';

  var bajaLex = lexs[0],
      MONTHS = [ 'january', 'february', 'march', 'april', 'may', 'june', 'july',
                 'august', 'september', 'october', 'november', 'december' ],
      WEEKDAYS = [ 'sunday', 'monday', 'tuesday', 'wednesday', 'thursday',
                   'friday', 'saturday' ];

  function get(k) {
    return bajaLex.get({ key: k, def: k });
  }
  function getShort(k) {
    return bajaLex.get({ key: k + '.short', def: k.substr(0, 3) });
  }
  function getMin(k) {
    return bajaLex.get({ key: k + '.short', def: k }).substr(0, 2);
  }

  /**
   * @private
   * @exports nmodule/webEditors/rc/fe/baja/util/localeUtils
   */
  var localeUtils = {};

  /**
   * Apply a limited subset of current lexicon info, using the user's current
   * language, into the Moment locale. Will also switch Moment into the current
   * locale, so locale-agnostic calls to `moment#format()` will use this locale
   * as well.
   *
   * @param {Function} moment Moment itself
   */
  localeUtils.applyMomentLocale = function (moment) {
    var lang = baja.getLanguage();

    moment.updateLocale(lang, {
      months: MONTHS.map(get),
      monthsShort: MONTHS.map(getShort),
      weekdays: WEEKDAYS.map(get),
      weekdaysShort: WEEKDAYS.map(getShort),
      weekdaysMin: WEEKDAYS.map(getMin)
    });

    moment.locale(lang);
  };

  return (localeUtils);
});
