/**
 * @copyright 2015 Tridium, Inc. All Rights Reserved.
 * @author Logan Byam
 */

/**
 * @private
 * @module nmodule/webEditors/rc/servlets/views
 */
define(['jquery', 'Promise'], function ($, Promise) {
  'use strict';

  var SERVLET_URI = '/view/all/ord/';

  function doGet(uri) {
    return Promise.resolve($.get(uri));
  }

  /**
   * Functions for retrieving information about what kinds of views are
   * available on a particular ORD.
   *
   * This module communicates with the following servlets:
   *
   * - com.tridium.web.servlets.ViewAllOrdServlet
   *
   * @private
   * @exports nmodule/webEditors/rc/servlets/views
   */
  var exports = {};

  /**
   * List the available views on this ord as an array of view objects.
   *
   * @param {baja.Ord|String} ord
   * @returns {Promise.<BajaViewInfo[]>} promise to be resolved with an
   * array of view objects
   */
  exports.listViews = function (ord) {
    return doGet(SERVLET_URI + ord)
      .then(function (obj) {
        return obj.list;
      });
  };

  return exports;
});
