var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * @copyright 2016 Tridium, Inc. All Rights Reserved.
 * @author Danesh Kamal
 */

/**
 * CompositeCommand
 * @module nmodule/webEditors/rc/wb/composite/commands/CompositeCommand
 * @private
 */
define(['baja!', 'baja!baja:CompositeAction,baja:CompositeTopic', 'jquery', 'lex!workbench', 'Promise', 'underscore', 'bajaux/commands/Command'], function (baja, types, $, lexs, Promise, _, Command) {

  'use strict';

  var IN = 0,
      OUT = 1,
      mapAsync = Promise.map,
      join = Promise.join,
      batchResolve = baja.BatchResolve.resolve;

  function createComposite(editor, params) {
    var composite = editor.value(),
        table = editor.$getTable(),
        model = table.value(),
        rows = model.getRows(),
        slotsToAdd = [],
        slotsToRename = [],
        slotsToReverse = [],
        slotsToRemove = editor.$removed || [];

    _(rows).each(function (row) {
      var slot = row.getSubject();
      if (!slot.backup) {
        slotsToAdd.push(slot);
      } else if (slot.dir === slot.backup.dir && slot.backup.name !== slot.name) {
        slotsToRename.push(slot);
      } else {
        slot.backup.backup = _.clone(slot.backup);
        slotsToReverse.push(slot);
      }
    });

    return Promise.join(addCompositeSlots(composite, slotsToAdd), renameCompositeSlots(composite, slotsToRename), reverseCompositeSlots(composite, slotsToReverse), removeCompositeSlots(composite, slotsToRemove)).then(function () {
      return reorderCompositeSlots(composite, rows);
    });
  }

  /////////////////////////////////////////////////////////////////////////////
  //Additions
  ////////////////////////////////////////////////////////////////////////////
  function addCompositeSlots(composite, slots) {
    var ords = _.map(slots, function (slot) {
      return slot.ord;
    }),
        promise = batchResolve({ ords: ords, base: composite, lease: true }).then(function (br) {
      var pairs = _.zip(br.getTargetObjects(), slots),
          linkPairs = _(pairs).filter(function (_ref) {
        var _ref2 = _slicedToArray(_ref, 2),
            target = _ref2[0],
            slot = _ref2[1];

        return slot.dir === OUT;
      }),
          knobPairs = _(pairs).filter(function (_ref3) {
        var _ref4 = _slicedToArray(_ref3, 2),
            target = _ref4[0],
            slot = _ref4[1];

        return slot.dir === IN;
      }),
          knobTargets = _(knobPairs).map(_.first),
          additionBatch = new baja.comm.Batch(),
          additionPromise = mapAsync(pairs, function (_ref5) {
        var _ref6 = _slicedToArray(_ref5, 2),
            child = _ref6[0],
            slot = _ref6[1];

        return addCompositeSlot(composite, child, slot, additionBatch);
      }).then(function () {

        //make links and knobs for the composite
        var makeConnectionsBatch = new baja.comm.Batch(),
            makeLinksPromise = mapAsync(linkPairs, function (_ref7) {
          var _ref8 = _slicedToArray(_ref7, 2),
              source = _ref8[0],
              slot = _ref8[1];

          return makeCompositeLink(composite, source, slot, makeConnectionsBatch);
        }),
            makeKnobsPromise = mapAsync(knobPairs, function (_ref9) {
          var _ref10 = _slicedToArray(_ref9, 2),
              target = _ref10[0],
              slot = _ref10[1];

          return makeCompositeKnob(target, composite, slot, makeConnectionsBatch);
        });

        makeConnectionsBatch.commit();

        return join(makeLinksPromise, makeKnobsPromise, function (incomingLinks, outgoingLinks) {

          //add links and knobs to the composite
          var addConnectionsBatch = new baja.comm.Batch(),
              targetLinkPairs = _.zip(knobTargets, outgoingLinks),
              addLinksPromise = mapAsync(incomingLinks, function (link) {
            return addLink(composite, link, addConnectionsBatch);
          }),
              addKnobsPromise = mapAsync(targetLinkPairs, function (_ref11) {
            var _ref12 = _slicedToArray(_ref11, 2),
                target = _ref12[0],
                link = _ref12[1];

            return addLink(target, link, addConnectionsBatch);
          });
          addConnectionsBatch.commit();
          return join(addLinksPromise, addKnobsPromise);
        });
      });

      additionBatch.commit();
      return additionPromise;
    });

    return promise;
  }

  function addCompositeSlot(composite, target, slot, batch) {
    var dir = slot.dir,
        name = slot.name,
        type = slot.type,
        targetSlot = target.getSlot(slot.slot),
        flags = dir === OUT ? slot.flags | baja.Flags.READONLY : slot.flags,
        facets = target.getFacets(targetSlot) || baja.Facets.NULL,
        value = void 0;


    if (targetSlot.isAction() || targetSlot.isTopic()) {
      value = dir === IN ? baja.$('baja:CompositeAction') : baja.$('baja:CompositeTopic');
    } else {
      value = type.getInstance();
    }

    //add the slot to the composite as part of a batch
    return composite.add({ slot: name, value: value, flags: flags, facets: facets, batch: batch });
  }

  function makeCompositeLink(target, source, slot, batch) {
    return target.makeLink({ source: source, sourceSlot: slot.slot, targetSlot: slot.name, batch: batch });
  }

  function makeCompositeKnob(target, source, slot, batch) {
    return target.makeLink({ source: source, sourceSlot: slot.name, targetSlot: slot.slot, batch: batch });
  }

  function addLink(target, link, batch) {
    return target.add({ value: link, facets: baja.Facets.NULL, flags: baja.Flags.COMPOSITE, batch: batch });
  }

  ////////////////////////////////////////////////////////////////////////////
  //Renames
  ////////////////////////////////////////////////////////////////////////////
  function renameCompositeSlots(composite, slots) {
    var ords = _(slots).map(function (slot) {
      return slot.ord;
    });
    return batchResolve({ ords: ords, base: composite, lease: true }).then(function (br) {
      var targets = br.getTargetObjects(),
          targetLinks = _.flatten(_.map(targets, function (target) {
        return target.getLinks();
      })),
          renameBatch = new baja.comm.Batch(),
          renamePromise = mapAsync(slots, function (slot) {
        var oldName = slot.backup.name,
            newName = slot.name;
        return composite.rename({ slot: oldName, newName: newName, batch: renameBatch }).then(function () {
          var updateBatch = new baja.comm.Batch(),
              links = _(composite.getLinks()).filter(function (link) {
            return link.getTargetSlotName() === oldName;
          }),
              knobs = _(targetLinks).filter(function (link) {
            return link.getSourceSlotName() === oldName;
          }),

          //update the link on the composite
          linksPromise = mapAsync(links, function (link) {
            return link.set({
              slot: 'targetSlotName',
              value: newName,
              batch: updateBatch
            });
          }),

          //update the link corresponding to the composite knob
          knobsPromise = mapAsync(knobs, function (link) {
            return link.set({
              slot: 'sourceSlotName',
              value: newName,
              batch: updateBatch
            });
          });

          updateBatch.commit();
          return join(linksPromise, knobsPromise);
        });
      });

      renameBatch.commit();
      return renamePromise;
    });
  }

  ////////////////////////////////////////////////////////////////////////////
  //Removals
  ////////////////////////////////////////////////////////////////////////////
  function removeCompositeSlots(composite, slots) {
    var targetToLinks = {},
        //want to use Map here but PhantomJS doesn't support it without a polyfill
    ords = _(slots).map(function (slot) {
      return slot.ord;
    });
    return batchResolve({ ords: ords, base: composite, lease: true }).then(function (br) {
      var targets = br.getTargetObjects(),
          removeBatch = new baja.comm.Batch(),
          removePromise = mapAsync(slots, function (slot) {
        if (slot.dir === OUT) {
          //remove links
          var removeLinksPromise = mapAsync(composite.getLinks(slot.name), function (link) {
            return composite.remove({
              slot: link.getPropertyInParent(),
              batch: removeBatch
            });
          });
          return join(removeLinksPromise, composite.remove({
            slot: slot.name,
            batch: removeBatch
          }));
        } else {
          //remove knobs
          _(targets).each(function (target) {
            targetToLinks[target] = _(target.getLinks()).filter(function (link) {
              return link.getSourceSlotName() === slot.name;
            });
          });
          return mapAsync(targets, function (target) {
            var targetLinks = targetToLinks[target],
                removeTargetLinksPromise = mapAsync(targetLinks, function (link) {
              return target.remove({
                slot: link.getPropertyInParent(),
                batch: removeBatch
              });
            });

            return join(removeTargetLinksPromise, composite.remove({
              slot: slot.name,
              batch: removeBatch
            }));
          });
        }
      });

      removeBatch.commit();
      return removePromise;
    });
  }

  function reverseCompositeSlots(composite, slots) {
    return removeCompositeSlots(composite, _(slots).map(function (slot) {
      return slot.backup;
    })).then(function () {
      return addCompositeSlots(composite, slots);
    });
  }

  function reorderCompositeSlots(composite, rows) {
    var list = _(rows).map(function (row) {
      return row.getSubject().name;
    }),
        slots = composite.getSlots().toArray();

    _(slots).each(function (slot) {
      var slotName = slot.getName();
      if (!(slot.isFrozen() || _(list).find(function (name) {
        return slotName === name;
      }))) {
        list.push(slotName);
      }
    });

    return composite.reorder(list);
  }

  /**
   * CompositeCommand
   *
   * @class CompositeCommand
   * @extends module:bajaux/commands/Command
   * @alias module:nmodule/webEditors/rc/wb/composite/commands/CompositeCommand
   * @private
   */

  var CompositeCommand = function (_Command) {
    _inherits(CompositeCommand, _Command);

    function CompositeCommand(editor) {
      _classCallCheck(this, CompositeCommand);

      return _possibleConstructorReturn(this, (CompositeCommand.__proto__ || Object.getPrototypeOf(CompositeCommand)).call(this, { func: function func(params) {
          return createComposite(editor, params);
        } }));
    }

    return CompositeCommand;
  }(Command);

  return CompositeCommand;
});
