function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * @copyright 2016 Tridium, Inc. All Rights Reserved.
 * @author Danesh Kamal
 */

/**
 * RenameCommand
 * @module nmodule/webEditors/rc/wb/composite/commands/RenameCommand
 * @private
 */

define(['baja!', 'jquery', 'lex!workbench', 'Promise', 'underscore', 'dialogs', 'nmodule/webEditors/rc/fe/feDialogs', 'bajaux/commands/Command'], function (baja, $, lexs, Promise, _, dialogs, feDialogs, Command) {

  'use strict';

  var lex = lexs[0],
      dialogTitle = lex.get("compositeEditor.rename.label"),
      errorMsg = lex.get("compositeEditor.rename.exists"),
      resolve = Promise.resolve,
      reject = Promise.reject;


  function showError(msg) {
    dialogs.showOk({
      title: dialogTitle,
      content: msg
    });
  }

  function handleRename(editor) {
    var composite = editor.value(),
        slots = composite.getSlots().toArray(),
        table = editor.$getTable(),
        selection = table.$getSelection(),
        model = table.value(),
        rows = model.getRows(),
        selectedRows = selection.getSelectedElements(model.getRows()),
        rowsChanged = [];

    function renameRows() {
      if (selectedRows.length > 0) {
        var row = selectedRows.shift(),
            slot = row.getSubject();
        return feDialogs.showFor({
          title: dialogTitle,
          value: slot.name,
          formFactor: 'mini'
        }).then(function (name) {
          if (name && name !== slot.name) {
            if (_(slots).some(function (slot) {
              return slot.getName() === name;
            }) || _(rows).some(function (row) {
              return row.getSubject().name === name;
            })) {
              return reject(errorMsg);
            } else {
              slot.name = name;
              rowsChanged.push(row);
            }
          }
        }).then(renameRows);
      }
      return resolve();
    }

    return renameRows().then(function () {
      return model.emit('rowsChanged', rowsChanged);
    }).catch(showError);
  }

  /**
   * RenameCommand
   *
   * @class MoveDownCommand
   * @extends module:bajaux/commands/Command
   * @alias module:nmodule/webEditors/rc/wb/composite/commands/RenameCommand
   * @private
   */

  var RenameCommand = function (_Command) {
    _inherits(RenameCommand, _Command);

    function RenameCommand(editor) {
      _classCallCheck(this, RenameCommand);

      return _possibleConstructorReturn(this, (RenameCommand.__proto__ || Object.getPrototypeOf(RenameCommand)).call(this, {
        module: "workbench",
        lex: 'compositeEditor.rename',
        enabled: false,
        func: function func() {
          return handleRename(editor);
        }
      }));
    }

    return RenameCommand;
  }(Command);

  return RenameCommand;
});
