/**
 * @copyright 2015 Tridium, Inc. All Rights Reserved.
 * @author Logan Byam
 */

/**
 * @private
 * @module nmodule/webEditors/rc/wb/profile/paletteSideBar
 */
define(['baja!',
        'bajaux/events',
        'Promise',
        'underscore',
        'nmodule/webEditors/rc/util/storageUtil',
        'nmodule/webEditors/rc/wb/shell/TitlePane',
        'nmodule/webEditors/rc/wb/tree/PaletteNavTree',
        'nmodule/webEditors/rc/servlets/palette'], function (
        baja,
        events,
        Promise,
        _,
        storageUtil,
        TitlePane,
        PaletteNavTree,
        palette) {

  'use strict';

  var getLocalStorage = storageUtil.getLocalStorage,

      INVOKE_EVENT = events.command.INVOKE_EVENT,

      DEFAULT_PALETTES = ['alarm', 'baja', 'control', 'history', 'kitControl',
        'schedule'];

  function paletteStorageKey() {
    return baja.getUserName() + '.profile.openPalettes';
  }

  function getOpenPalettes() {
    try {
      var openPalettes = getLocalStorage().getItem(paletteStorageKey());
      if (openPalettes) {
        return JSON.parse(openPalettes);
      }
    } catch (e) {}

    return DEFAULT_PALETTES;
  }

  function savePalettes(paletteNames) {
    try {
      getLocalStorage().setItem(paletteStorageKey(), JSON.stringify(paletteNames));
    } catch (e) {}
  }

  /**
   * Loads the Palette Sidebar.
   *
   * @inner
   * @private
   *
   * @param {JQuery} dom The jQuery DOM wrapper for the DOM to load the Sidebar into.
   */
  return function load(dom) {

    dom.one("sidebar-layout", function () {
      var titlePane = new TitlePane(),
          navTree = new PaletteNavTree(),
          moduleNames = getOpenPalettes();

      Promise.join(palette.getPalettes(moduleNames), titlePane.initialize(dom))
        .spread(function (paletteNode) {

          dom.on(INVOKE_EVENT, function (e, cmd) {
            navTree.value().getKids()
              .then(function (kids) {
                var openPalettes = _.map(kids, function (kid) {
                  return kid.getName();
                });
                savePalettes(openPalettes);
              });
          });

          return titlePane.setContentWidget(navTree)
            .then(function () {
              return titlePane.load(paletteNode);
            });
        });
    });
  };
});
