/**
 * @copyright 2015 Tridium, Inc. All Rights Reserved.
 * @author Gareth Johnson.
 */

/*jshint browser: true*/

/**
 * Loads the Search Sidebar.
 *
 * @private
 * @module nmodule/webEditors/rc/wb/profile/searchSideBar
 */
define(['baja!',
        'log!nmodule.webEditors.rc.wb.profile.searchSideBar',
        'Promise',
        'nmodule/webEditors/rc/fe/fe',
        "nmodule/webEditors/rc/wb/shell/TitlePane"], function (
        baja,
        log,
        Promise,
        fe,
        TitlePane) {

  'use strict';

  var logError = log.severe.bind(log);

  /**
   * Loads the Search Sidebar.
   *
   * @inner
   * @private
   *
   * @param  {JQuery} dom The jQuery DOM wrapper for the DOM to load the Sidebar into.
   */
  return function load(dom) {
    dom.one("sidebar-layout", function () {
      var titlePane = new TitlePane();
      return Promise.join(
        fe.makeFor({
          moduleName: 'search',
          keyName: 'SearchWidget',
          type: 'search:SearchWidget',
          formFactor: 'compact'
        }),
        baja.Ord.make('service:search:SearchService').get(),
        titlePane.initialize(dom)
      )
        .spread(function (searchWidget, searchService) {
          return titlePane.setContentWidget(searchWidget)
            .then(function() {
              return searchWidget.load(searchService);
            });
        })
        .catch(logError);
    });
  };
});
