/**
 * @copyright 2015 Tridium, Inc. All Rights Reserved.
 * @author Logan Byam
 */

/**
 * @private
 * @module nmodule/webEditors/rc/wb/shell/TitlePane
 */
define(['Promise',
        'bajaux/Widget',
        'bajaux/util/CommandButtonGroup',
        'nmodule/webEditors/rc/fe/fe',
        'hbs!nmodule/webEditors/rc/wb/shell/template/TitlePane'], function (
         Promise,
         Widget,
         CommandButtonGroup,
         fe,
         tplTitlePane) {

  'use strict';

  var TitlePane = function TitlePane() {
    Widget.apply(this, arguments);
  };
  TitlePane.prototype = Object.create(Widget.prototype);
  TitlePane.prototype.constructor = TitlePane;

  TitlePane.prototype.$getDisplayNameElement = function () {
    return this.jq().children('.header').find('.tab');
  };

  TitlePane.prototype.$getCommandsElement = function () {
    return this.jq().children('.header').find('.commands');
  };

  TitlePane.prototype.$getControlsElement = function () {
    return this.jq().children('.header').find('.controls');
  };

  TitlePane.prototype.$getCommandButtonGroup = function () {
    return this.$getCommandsElement().data('widget');
  };

  TitlePane.prototype.$getContentElement = function () {
    return this.jq().children('.content');
  };

  TitlePane.prototype.doInitialize = function (dom) {
    if (!dom.hasClass('TitlePane')) {
      dom.html(tplTitlePane({}));
      dom.addClass('TitlePane');
    }
  };

  TitlePane.prototype.doLoad = function (value) {
    var widget = this.getContentWidget();

    if (!widget) {
      return Promise.reject(new Error('no content widget set'));
    }

    return widget.load(value);
  };

  TitlePane.prototype.getContentWidget = function () {
    var contentElement = this.$getContentElement();

    //TODO: because the shell profile loads an entire hx view into the content element, the actual widget gets instantiated a couple levels down.
    return Widget.in(contentElement) ||
      Widget.in(contentElement
        .children('.bajaux-container')
        .children('.bajaux-widget-container')
        .children('.bajaux-widget'));
  };

  /**
   * @param {module:bajaux/Widget} widget
   * @returns {Promise}
   */
  TitlePane.prototype.setContentWidget = function (widget) {
    var that = this,
        oldWidget,
        oldButtons = that.$getCommandButtonGroup();

    if (!widget) {
      return Promise.reject(new Error('widget required'));
    }

//    if (widget.jq()) {
//      return Promise.reject(new Error('content widget already initialized elsewhere'));
//    }

    oldWidget = that.getContentWidget();

    return Promise.join(
        oldWidget && (oldWidget !== widget) && oldWidget.destroy(),
        oldButtons && oldButtons.destroy()
      )
      .then(function () {
        return Promise.join(
          widget.toDisplayName(),
          !widget.jq() && widget.initialize(that.$getContentElement()),
          fe.buildFor({
            value: widget.getCommandGroup(),
            dom: that.$getCommandsElement(),
            type: CommandButtonGroup,
            properties: { toolbar: true }
          })
        );
      })
      .spread(function (displayName) {
        that.$getDisplayNameElement().text(displayName);
      });
  };

  TitlePane.prototype.doDestroy = function () {
    var that = this,
        content = that.getContentWidget();

    that.jq().removeClass('TitlePane');

    return Promise.resolve(content && content.destroy());
  };

  return TitlePane;
});
