/**
 * @copyright 2015 Tridium, Inc. All Rights Reserved.
 * @author Logan Byam
 */

/**
 * @private
 * @module nmodule/webEditors/rc/wb/table/menu/ShowHideColumnsContextMenu
 */
define(['bajaux/commands/Command',
        'bajaux/commands/CommandGroup',
        'underscore',
        'Promise',
        'nmodule/webEditors/rc/wb/menu/CommandGroupContextMenu'], function (
         Command,
         CommandGroup,
         _,
         Promise,
         CommandGroupContextMenu) {

  'use strict';

  var CHECK_ICON = 'module://icons/x16/whiteIcons/check.png';

  var ShowHideColumnsContextMenu = function ShowHideColumnsContextMenu(table) {
    this.$table = table;
  };
  ShowHideColumnsContextMenu.prototype = Object.create(CommandGroupContextMenu.prototype);
  ShowHideColumnsContextMenu.prototype.constructor = ShowHideColumnsContextMenu;

  ShowHideColumnsContextMenu.prototype.$getClassName = function () {
    return '';
  };

  ShowHideColumnsContextMenu.prototype.toContextMenu = function () {
    var table = this.$table,
        model = table.value(),
        columns = _.filter(model.getColumns(), function (col) { return col.isHidable(); });

    return Promise.all(columns.map(function (column) {
      return Promise.resolve(column.toDisplayName())
        .then(function (displayName) {
          return new Command({
            displayName: displayName,
            icon: column.isUnseen() ? null : CHECK_ICON,
            func: function () {
              column.setUnseen(!column.isUnseen());
            }
          });
        });
    }))
      .then(function (commands) {
        return new CommandGroup({ commands: commands });
      });
  };

  ShowHideColumnsContextMenu.prototype.arm = function (jq, selector) {
    var that = this;

    jq.on('click', selector, function (e) {
      return that.$doContextMenu(e);
    });

    that.$armContextMenuAsync(jq, selector);
  };

  return ShowHideColumnsContextMenu;
});
