/**
 * @copyright 2015 Tridium, Inc. All Rights Reserved.
 * @author Logan Byam
 */

/**
 * @module nmodule/webEditors/rc/wb/table/model/Row
 */
define(['underscore'], function (_) {
  'use strict';

  /**
   * API Status: **Development**
   *
   * Row for use in a `TableModel`.
   *
   * @class
   * @alias module:nmodule/webEditors/rc/wb/table/model/Row
   * @param {*} subject the value represented by this row
   * @param {*} [icon]
   * @see module:nmodule/webEditors/rc/wb/table/model/TableModel
   */
  var Row = function Row(subject, icon) {
    this.$subject = subject;
    this.$icon = icon || null;
    this.$data = {};
  };

  /**
   * Get the value represented by this row.
   *
   * @returns {*}
   */
  Row.prototype.getSubject = function () {
    return this.$subject;
  };

  /**
   * Update the value of the row.
   *
   * This is currently intended for internal use by the framework only.
   * @private
   * @param {*} subject
   */
  Row.prototype.$setSubject = function (subject) {
    this.$subject = subject;
  };

  /**
   * Get this row's icon.
   *
   * @returns {*} the icon, or null if none given
   */
  Row.prototype.getIcon = function () {
    return this.$icon;
  };

  /**
   * Allow arbitrary metadata to be set temporarily on the row or retrieved.
   *
   * @param {String} key
   * @param {*} [value]
   * @returns {*} `undefined` if used as a setter; data value if used as a
   * getter
   */
  Row.prototype.data = function (key, value) {
    var data = this.$data;
    if (arguments.length === 1) {
      return data[key];
    } else {
      data[key] = value;
      return value;
    }
  };

  /**
   * Return an array of key names for the metadata values set on the row
   * via the `data()` method.
   *
   * @returns {Array.<String>}
   */
  Row.prototype.getDataKeys = function () {
    return _.keys(this.$data);
  };

  /**
   * Delete the metadata value with the given key name.
   *
   * @param {String} key
   */
  Row.prototype.deleteData = function (key) {
    delete this.$data[String(key)];
  };

  return Row;
});
